/*
 *  comm.c - Functions handling communication between main- and player-process.
 *
 *  (C) 1994, 1995 Mikael Nordqvist (d91mn@efd.lth.se, mech@df.lth.se)
 */

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <errno.h>
#include <time.h>
#include <sys/soundcard.h>
#include <dirent.h>

#include "mod.h"
#include "message.h"

/* External variables */

SEQ_DECLAREBUF();


/* Mainloop. Never returns. */

void main_loop(void)
{
    fd_set rset;
    int nr;

#ifdef COMPILE_GUI_TK
    /* Enter TK's mainloop if we have GUI_TK */
    if(GUI_mode == GUI_TK)
	tk_call_mainloop();
#endif
    while(!quit) {
	FD_ZERO(&rset);
	if(!quiet)
	    FD_SET(STDIN_FILENO, &rset);
	if(loaded)
	    FD_SET(seqfd, &rset);
	if(!suspend) {
	    if((nr=select(maxfd, &rset, 0, 0, 0)) >= 0) {
		if(FD_ISSET(STDIN_FILENO, &rset))
		    read_key();
		else if(FD_ISSET(seqfd, &rset))
		    read_sequencer();
		if(!nr)
		    warning("nr == 0 ?!?!?\n");
	    }
	    else {
		if(errno != EINTR)
		    error("select() failed (errno=%d).\n", errno);
	    }
	}
#if 0
	else {
	    screen_suspend();
	    debug("\na\nb\nSuspending...\n");
	    suspend_program();
	    screen_resume();
	    debug("\na\nb\nOk, resumed...\n");
	}
#endif
    }
    cleanup();
    exit(0);
}


void read_key(void)
{
#ifdef COMPILE_GUI_NC 
    if(GUI_mode == GUI_NC)
	nc_read_key();
#endif
    if(GUI_mode == GUI_SIMPLE)
	simple_read_key();
}


void read_sequencer(void)
{
    char buf[4];
    int msg;

    debug("SR: read_seq... ");
    safe_read(seqfd, (struct mod_message *)buf, 4);
    msg=(*(unsigned long *)buf)>>8;
    debug("SR: done... (%d) ", msg);
    switch(msg) {
      case MESSAGE_PRINTLINE:
      case MESSAGE_PRINTSPEED:
	if(!maxtime_done && opt.maxtime &&
	   time(0)-starting_time == opt.maxtime) {
	    print_time(opt.maxtime);
	    command_player(MESSAGE_STOP, 42);
	    maxtime_done=1;
	    goto overtime;
	}
	read_syncedmessage();
	break;
      case MESSAGE_DONE: /* Song played to the end */
	total_time+=time(0)-restart_time;
	restart_time=time(0);
      overtime:
	set_playing(0);
	last_line=0;
	last_pos=opt.start_pos;
	print_pos(last_pos, 0);
	debug("MESS_DONE, calling end_of_song()...\n");
	end_of_song();
	debug("end_of_song() done.\n");
	break;
      default:
	error("Invalid message %d\n", msg);
    }
}


void read_syncedmessage(void)
{
    struct mod_message mess;
    
    debug("SR: read_syncseq... ");
    safe_read(player2main_synced[PIPE_READ], &mess, sizeof(mess));
    debug("SR: done... ");
    switch(mess.type) {
      case MESSAGE_PRINTSPEED:
	last_speed=mess.a1.speed;
	last_tempo=mess.a2.tempo;
	print_speed(last_speed, last_tempo);
	break;
      case MESSAGE_PRINTLINE:
	last_pos=mess.a1.songpos;
	last_line=mess.a2.line;
	print_line(last_pos, last_line, pending_syncedmessage());
	print_pos(last_pos, last_line);
	print_time(time(0)-starting_time);
	break;
      default:
	error("Illegal synced message received by main (%d).\n", mess.type);
    }
}


int pending_syncedmessage(void)
{
    fd_set rset;
    struct timeval null_time={0, 0};

    FD_ZERO(&rset);
    FD_SET(seqfd, &rset);
    if(select(seqfd+1, &rset, 0, 0, &null_time) >= 0) {
	return FD_ISSET(seqfd, &rset);
    }
    else {
	if(errno != EINTR)
	    error("select() failed (errno=%d).\n", errno);
    }
    return 0; /* Remove warning */
}


void command_player(int command, int arg)
{
    struct mod_message mess;
    
    if(playing && !command == MESSAGE_UPDATEOPTIONS)
	total_time+=time(0)-restart_time;
    
    mess.type=command;
    switch(command) {
      case MESSAGE_EXIT:
	send_to_player(&mess);
	set_playing(0);
	break;
      case MESSAGE_STOP:
	send_to_player(&mess);
	set_playing(0);
	last_line=0;
	wait_ack();
	break;
      case MESSAGE_JUMPPATTERN:
	mess.a1.songpos=arg;
	send_to_player(&mess);
	mess.a1.speed=last_speed;
	mess.a2.tempo=last_tempo;
	send_to_player(&mess);
	send_to_player_generic(&opt.active_voices, sizeof(opt.active_voices));
	last_pos=arg;
	last_line=0;
	wait_ack();
	if(!playing) {
	    starting_time=time(0)-(restart_time-starting_time);
	}
	set_playing(1);
	break;
      case MESSAGE_UPDATEOPTIONS:
	send_to_player(&mess);
	send_to_player_generic(&opt, sizeof(opt));
	break;
      default:
	error("Internal error in handle_restart()");
    }
    restart_time=time(0);
}

/* Should only becalled just after a MESSAGE_JUMPPATTERN has been sent */

void reset_playingtime(void)
{
    starting_time=restart_time=time(0);
}

/* If outgoing_too is false this function will hang until somone drains
 * the main2player-pipe (busy-wait, but this should only be for very
 * short periods of time if everything is working as it should).
 */

void drain_pipes(char outgoing_too)
{
    fd_set rset;
    char buf[256];
    int nr;
    struct timeval nulltime;

    while(1) {
	nulltime.tv_sec=0;
	nulltime.tv_usec=0;
	FD_ZERO(&rset);

	FD_SET(main2player[PIPE_READ], &rset);
	FD_SET(player2main_synced[PIPE_READ], &rset);
	FD_SET(seqfd, &rset);
	if(!quiet && GUI_mode != GUI_TK)
	    FD_SET(STDIN_FILENO, &rset);

        if((nr=select(maxfd, &rset, 0, 0, &nulltime)) > 0) {
	    if(outgoing_too && FD_ISSET(main2player[PIPE_READ], &rset))
		read(main2player[PIPE_READ], buf, 256);
	    if(FD_ISSET(player2main_synced[PIPE_READ], &rset))
		read(player2main_synced[PIPE_READ], buf, 256);
	    if(FD_ISSET(seqfd, &rset))
		read(seqfd, buf, 256);
	    if(FD_ISSET(STDIN_FILENO, &rset))
		read(STDIN_FILENO, buf, 256);
	}
	else {
	    if(nr < 0)
		error("Select problems (%d).\n", errno);
	    break; /* Done */
	}
    }
}


void send_to_player(struct mod_message *m)
{
    safe_write(main2player[PIPE_WRITE], m, sizeof(struct mod_message));
}


void send_to_player_generic(void *m, int size)
{
    safe_write(main2player[PIPE_WRITE], (struct mod_message *)m, size);
}


/* Handshake-functions */

void send_ack(void)
{
    char data;
    debug("Start send ACK...\n");
    safe_write(ack_pipe[PIPE_WRITE], (struct mod_message *)&data, 1);
    debug("ACK sent.\n");
}


void wait_ack(void)
{
    char data;
    debug("Wait ACK...\n");
    safe_read(ack_pipe[PIPE_READ], (struct mod_message *)&data, 1);
    debug("Got ACK.\n");
}
