/*
 *  file.c - Handles directory browsing.
 *
 *  (C) 1994, 1995 Mikael Nordqvist (d91mn@efd.lth.se, mech@df.lth.se)
 */

#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "mod.h"

/* Alpha-sort + directories at the top (with '../' last) */

int my_alphasort(const struct dirent * const *a,
		 const struct dirent * const *b)
{
    struct stat sa, sb;

    if(!stat((*a)->d_name, &sa) &&
       !stat((*b)->d_name, &sb)) {
	if((S_ISDIR(sa.st_mode) && S_ISDIR(sb.st_mode)) ||
	   (!S_ISDIR(sa.st_mode) && !S_ISDIR(sb.st_mode))) {
#if 0
	    /* Make sure .. is last of the directories */
	    if(!strcmp("..", (*a)->d_name))
		return 1;
	    else if(!strcmp("..", (*b)->d_name))
		return -1;
	    else
#endif
		return alphasort((void *)a, (void *)b);
	}
	else {
	    if(S_ISDIR(sa.st_mode))
		return -1;
	    else
		return 1;
	}
    }
    else {
	error("Stat failed\n");
	/* return alphasort((void *)a, (void *)b); */
    }
    return 0; /* Never reached */
}


int choose_file(const struct dirent *de)
{
    struct stat sa;
    
    if(!strcmp("/", workdir) && !strcmp("..", de->d_name))
	return 0;

    /* Here we could weed out files with incorrect suffixes */
    
    if(!strcmp(".", de->d_name) || stat(de->d_name, &sa)) {
	/* Restore errno as this function is called from within libc, and any
	 * changes to errno (from a failed stat() will affect scandir().
	 */
	errno=0;
	return 0;
    }
    else
	return 1;
}


/* Read directory */

void init_dir(void)
{
    int i, tmp;
    struct stat s;
    struct dirent **tmp_files;
    char *name;

    getcwd(workdir, PATH_MAX);
    
    if(files)
	free(files);
    if(file_names)
	free(file_names);
    
    /* Should always be at least 1 file (..) */
    if((nr_files=scandir(workdir, &tmp_files, choose_file, my_alphasort)) == -1)
	error("scandir() failed (%d).\n", errno);
    
    files=(struct mod_file *)malloc(sizeof(struct mod_file)*nr_files);
    
    for(tmp=i=0; i < nr_files; ++i)
	tmp+=1+strlen(tmp_files[i]->d_name);
    
    file_names=name=(char *)malloc(tmp);
    
    nr_dirs=0;
    for(i=0; i < nr_files; ++i) {
	strcpy(name, tmp_files[i]->d_name);
	free(tmp_files[i]);
	files[i].name=name;
	if(stat(name, &s)) {
	    files[i].size=-1;  /* Invalid file */
	    files[i].dir=0;
	}
	else {
	    if(S_ISDIR(s.st_mode)) {
		files[i].dir=1;
		nr_dirs++;
	    }
	    else
		files[i].dir=0;
	    
	    files[i].size=s.st_size;
	}
	name+=1+strlen(name);
    }
    free(tmp_files);
}


void remove_file_nr(int nr)
{
    if(files[nr].dir)
	error("Trying to remove a directory.\n");
    if(nr != nr_files-1) {
	memmove(&files[nr], &files[nr+1],
		(nr_files-nr-1)*sizeof(struct mod_file));
    }
    --nr_files;
}


/* This function should be moved to nc_screen.c */

char *get_filestring(int nr)
{
    static char buf[40], *p;

    if(nr == -1) {
	strcpy(buf, "                                     ");
	return buf;
    }

    if(files[nr].size >= 0) {
	if(!files[nr].dir)
	    sprintf(buf, "%7d   ", files[nr].size);
	else
	    strcpy(buf, "          ");
	
	strcpy(&buf[10], fill_string(files[nr].name, files[nr].dir ? 26 : 27));
	
	if(files[nr].dir) {
	    p=&buf[36];
	    *(p+1)=0;
	    *p=' ';
	    while(*p == ' ')
		p--;
	    *(++p)='/';
	}
    }
    else {
	strcpy(buf, "      -   ");
	strcpy(&buf[10], fill_string(files[nr].name, 27));
    }
    
    return buf;
}


char *get_filename(int nr)
{
    return files[nr].name;
}


int is_dir(int nr)
{
    return files[nr].dir;
}

int is_dir_name(char *name)
{
    struct stat sa;
    
    if(!stat(name, &sa) && S_ISDIR(sa.st_mode))
	return 1;
    else
	return 0;
}

/* Returns the filenumber of the specified directory name if it exists,
 * otherwise zero.
 */

int name_to_filenr(char *name)
{
    int i;
    for(i=0; i < nr_files; ++i)
	if(!strcmp(name, files[i].name) && files[i].dir)
	    return i;
    return 0;
}
