/*
 *  mixer.c - Handles master-volume and muting.
 *
 *  (C) 1994, 1995 Mikael Nordqvist (d91mn@efd.lth.se, mech@df.lth.se)
 */

#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>

#include "mod.h"

static int volume=100, mute=0;

void init_mixer(void)
{
    int mixer_devmask;
    
    if((mixerfd=open(mixer_name, O_RDWR, 0)) != -1) {
	if(!(ioctl(mixerfd, SOUND_MIXER_READ_DEVMASK, &mixer_devmask) != -1 &&
	     (mixer_devmask & SOUND_MASK_SYNTH)))
	{
	    close(mixerfd);
	    mixerfd=-1;
	}
    }
    get_mastervolume(); /* Make sure mixer is set correctly */
}


void cleanup_mixer(void)
{
    if(mixerfd != -1)
	close(mixerfd);
}


int mixer_available(void)
{
    return mixerfd != -1;
}

static int get_vol(void)
{
    int current;
    
    if(ioctl(mixerfd, SOUND_MIXER_READ_SYNTH, &current) == -1)
	return 100;
    
    /* Take average of left/right */
    return ((current&0xff)+((current>>8)&0xff))/2;
}

int get_mastervolume(void)
{
    if(mixerfd == -1)
	return 100;
    
    if(mute)
	return volume;
    volume=get_vol();

    return volume;
}


void set_mastervolume(int value)
{
    if(mixerfd == -1)
	return;
    
    volume=value;

    if(mute)
	mute=0;

    value=value|(value<<8);
    ioctl(mixerfd, SOUND_MIXER_WRITE_SYNTH, &value);
}

void change_mastervolume(int amount)
{
    if(mute)
	mute=0;
    
    set_mastervolume(MIN(MAX(0, volume+amount), 100));
}

void sync_volume(void)
{
    int v;

    v=get_vol();
    if(v) {
	mute=0;
	volume=v;
    }
}

void set_mute(int set_mute)
{
    int zero=0;
    
    if(mixerfd == -1)
	return;
    
    if(set_mute) {
	mute=1;
	ioctl(mixerfd, SOUND_MIXER_WRITE_SYNTH, &zero);
    }
    else {
	mute=0;
	set_mastervolume(volume);
    }
}


void toggle_mute(void)
{
    if(mixerfd != -1)
	set_mute(!mute);
}

int is_mute(void)
{
    return mute;
}
