/*
 *  nc_controller.c - Controller for the ncurses interface.
 *
 *  (C) 1994, 1995 Mikael Nordqvist (d91mn@efd.lth.se, mech@df.lth.se)
 */

#ifdef COMPILE_GUI_NC

#include <unistd.h>
#include <string.h>
#include <time.h>
#include <ncurses.h>
#include <errno.h>
#include <fcntl.h>

#include "mod.h"
#include "message.h"

extern int swin_LINES;
extern int first_voice, cur_voice, nr_visible_voices, first_sample;

void nc_init_controller(void)
{
    selecting=0;
    cur_filenr=cur_listnr=0;

    L_order_reset(PL);
    /* No exit after last, no persistency */
    if(!(default_opt.auto_next && L_size(PL) && load_next(0, 0, 0)))
	nc_process_key((int)'l');
}


void nc_end_of_song(void)
{
    if(!selecting)
	nc_clear_screen();
    if(!selecting && default_opt.auto_next && loaded == LOADED_LIST)
	load_next(0, 0, 0); /* No exit after last, no persistency */
}


void nc_read_key(void)
{
    nc_process_key(getch());
}


void nc_process_key(int ch)
{
    int tmp, fd;
    char dirname[PATH_MAX+1], buf[PATH_MAX+1];
    char dotdot=0; /* Remove warning */
    static int last_selecting=SELECTING_LOAD;
    
    /* Keys always available */
    switch(ch) {
      case '?':
      case KEY_F(1):
      case 'h':
	nc_print_minihelp(1);
	break;
      case 'H':
	nc_print_minihelp(-1);
	break;
      case 'q':
	if(loaded)
	    command_player(MESSAGE_EXIT, 42);
        quit=QUIT_QUIT;
	break;
      case '+':
	sync_volume();
	change_mastervolume(5);
	nc_print_volume();
	break;
      case '-':
	sync_volume();
	change_mastervolume(-5);
	nc_print_volume();
	break;
      case 'm':	
	sync_volume();
	toggle_mute();
	nc_print_volume();
	break;
      case '>':
	nc_inc_swin();
	break;
      case '<':
	nc_dec_swin();
	break;
      case 12: /* Ctrl-L */
	nc_touch_windows();
	nc_refresh_screen(1);
	break;
      case 'b':
	if(default_opt.browse)
	    default_opt.browse=0;
	else
	    default_opt.browse=1;
	nc_print_flags();
	break;
      case 's':
	if(default_opt.noscroll) {
	    default_opt.noscroll=0;
	    if(playing && !selecting)
		nc_print_line(last_pos, last_line);
	}
	else {
	    default_opt.noscroll=1;
	    if(playing && !selecting)
		nc_clear_screen();
	}
	nc_print_flags();
	break;
      case 'r':
	if(default_opt.random_mode)
	    default_opt.random_mode=0;
	else
	    default_opt.random_mode=1;
	
	L_order_reset(PL);
	if(loaded == LOADED_LIST) {
	    tmp=L_order_name_linearget(PL, M.real_filename);
	    if(default_opt.random_mode)
		L_order_linear_swapfirst(PL, tmp);
	    if(tmp == -1)
		debug("Something wrong here...\n");
	    L_order_linearset(PL, tmp);
	}
	nc_print_flags();
	break;
      default:
    }

    if(selecting) {
        /* Keys independent of load/list mode */
	switch(ch) {
	  case 'l':
	    last_selecting=selecting;
	    selecting=0;
	    nc_print_channelnumbers();
	    if(playing && !default_opt.noscroll)
		nc_print_line(last_pos, last_line);
	    else
		nc_clear_screen();
	    break;
	  case 'S':
	    if(!L_size(PL))
		break;
	    if(!*playlist_name) {
		strcpy(dirname, "/tmp/mod.lst");
		for(tmp=1; ; ++tmp) {
		    if((fd=open(dirname, O_RDONLY)) == -1)
			break;
		    close(fd);
		    sprintf(dirname, "/tmp/mod%d.lst", tmp);
		}
		strcpy(buf, "New list: ");
		strcat(buf, dirname);
		print_status(buf);
		sleep(1);
		strcpy(playlist_name, dirname);
	    }
	    if(!save_playlist(playlist_name, PL))
		print_status("Error saving list!!!");
	    else {
		sprintf(buf, "Saved '%s'.", path_basename(playlist_name, 0));
		print_status(buf);
	    }
	    sleep(1);
	    set_playing(playing);
	    break;
	  default: /* All other keys ignored */
	}
	
	if(selecting == SELECTING_LOAD) {
	    switch(ch) {
	      case KEY_UP:
	      case 'p':
		cur_filenr=MAX(cur_filenr-1, 0);
		nc_print_files(cur_filenr);
		break;
	      case KEY_DOWN:
	      case 'n':
		cur_filenr=MIN(cur_filenr+1, nr_files-1);
		nc_print_files(cur_filenr);
		break;
	      case KEY_PPAGE:
	      case 'P':
		cur_filenr=MAX(cur_filenr-swin_LINES/2, 0);
		nc_print_files(cur_filenr);
		break;
	      case KEY_NPAGE:
	      case 'N':
		cur_filenr=MIN(cur_filenr+swin_LINES/2, nr_files-1);
		nc_print_files(cur_filenr);
		break;
	      case KEY_HOME:
		cur_filenr=0;
		nc_print_files(cur_filenr);
		break;
	      case KEY_END:
		cur_filenr=nr_files-1;
		nc_print_files(cur_filenr);
		break;
	      case '\n':
		if(!is_dir(cur_filenr)) {
		    strcpy(dirname, workdir);
		    strcat(dirname, "/");
		    strcat(dirname, get_filename(cur_filenr));
		    if(strcmp(".lst", &dirname[strlen(dirname)-4])) {
			if(change_module(dirname, "")) {
			    loaded=LOADED_FILE;
			    loaded_filenr=cur_filenr;
			    if(!default_opt.browse) {
				last_selecting=selecting;
				selecting=0;
			    }
			}
			else {
			    loaded=0;
			    nc_print_files(cur_filenr);
			    nc_print_list(cur_listnr);
			}
			set_playing(playing);
		    }
		    else { /* Load list */
			print_status("Loading list...");
			if(!load_playlist(dirname, PL)) {
			    print_status("Error loading list...");
			    sleep(1);
			}
			else {
			    strcpy(playlist_name, dirname);
			    L_order_reset(PL);
			    if(loaded == LOADED_LIST)
				loaded=LOADED_VOID;
			    cur_listnr=0;
			    nc_print_files(cur_filenr);
			    nc_print_list(cur_listnr);
			}
			set_playing(playing);
		    }
		}
		else {
		    nc_clear_left_screen();
		    if(!strcmp("..", get_filename(cur_filenr))) {
			strcpy(dirname, path_basename(workdir, 0));
			dotdot=1;
		    }
		    change_dir(get_filename(cur_filenr));
		  yucky_goto:
		    print_status("Loading directory...");
		    init_dir();
		    set_playing(playing); /* Restore status-line */
		    cur_filenr=(dotdot ? name_to_filenr(dirname) : 0);
		    nc_print_files(cur_filenr);
		    nc_print_list(cur_listnr);
		    nc_print_selectingbar();
		}
		break;
	      case 'u':
		if(!strcmp("/", workdir))
		    break;
		strcpy(dirname, path_basename(workdir, 0));
		dotdot=1;
		change_dir("..");
		goto yucky_goto;
		break;
	      case KEY_RIGHT:
	      case '\t':
		if(L_size(PL)) {
		    selecting=SELECTING_LIST;
		    nc_print_selectingbar();
		    nc_print_list(cur_listnr);
		}
		break;
	      case KEY_IC:
	      case ' ':
		ch='t'; /* Fall through to 't' */
	      case 't': /* Fall through to 'T' */
	      case 'T':
		if(is_dir(cur_filenr))
		    break;	
		strcpy(dirname, workdir);
		strcat(dirname, "/");
		strcat(dirname, get_filename(cur_filenr));
		if(ch == 't')
		    L_insert_after_nr(PL, cur_listnr, dirname, "");
		else
		    L_insert_before_nr(PL, cur_listnr, dirname, "");
		L_order_reset(PL);
		if(loaded == LOADED_LIST) {
		    tmp=L_order_name_linearget(PL, M.real_filename);
		    if(tmp == -1)
			debug("Something wrong here...\n");
		    L_order_linearset(PL, tmp);
		}
		
		if(L_size(PL) > 1 && ch == 't')
		    ++cur_listnr;
		nc_print_list(cur_listnr);
		cur_filenr=MIN(cur_filenr+1, nr_files-1);
		nc_print_files(cur_filenr);
		break;
	      case 'D':
		if(!default_opt.allow_delete || is_dir(cur_filenr))
		    break;
		strcpy(dirname, workdir);
		strcat(dirname, "/");
		strcat(dirname, get_filename(cur_filenr));
		
		if(!unlink(dirname)) { /* Delete successful */
		    if(loaded == LOADED_FILE)
			loaded=LOADED_VOID;
		    remove_file_nr(cur_filenr);
		    cur_filenr=MIN(cur_filenr, nr_files-1);
		    nc_print_files(cur_filenr);
		    nc_print_list(cur_listnr);
		    nc_print_selectingbar();
		}
		break;
	      default: /* All other keys ignored */
	    }
	}
	else if (selecting == SELECTING_LIST) {
	    switch(ch) {
	      case KEY_UP:
	      case 'p':
		cur_listnr=MAX(cur_listnr-1, 0);
		nc_print_list(cur_listnr);
		break;
	      case KEY_DOWN:
	      case 'n':
		cur_listnr=MIN(cur_listnr+1, L_size(PL)-1);
		nc_print_list(cur_listnr);
		break;
	      case KEY_PPAGE:
	      case 'P':
		cur_listnr=MAX(cur_listnr-swin_LINES/2, 0);
		nc_print_list(cur_listnr);
		break;
	      case KEY_NPAGE:
	      case 'N':
		cur_listnr=MIN(cur_listnr+swin_LINES/2, L_size(PL)-1);
		nc_print_list(cur_listnr);
		break;
	      case KEY_HOME:
		cur_listnr=0;
		nc_print_list(cur_listnr);
		break;
	      case KEY_END:
		cur_listnr=L_size(PL)-1;
		nc_print_list(cur_listnr);
		break;
	      case KEY_LEFT:
	      case '\t':
	      ugly_goto:
		selecting=SELECTING_LOAD;
		nc_print_selectingbar();
		nc_print_files(cur_filenr);
		break;
	      case '\n':
		if(!L_size(PL))
		    break;

		if(default_opt.random_mode) {
		    L_order_reset(PL);
		    L_order_linear_swapfirst(PL, cur_listnr);
		    if(loaded==LOADED_FILE)
			loaded=LOADED_VOID; /* Make sure load_next()
					     * starts the list
					     */
		    load_next(0, 0, 0); /* No exit after last,
					 * no persistency
					 */
		}
		else {
		    L_order_linearset(PL, cur_listnr);
		    if(change_module(PL->current->filename,
				     PL->current->options))
			loaded=LOADED_LIST;
		}
		if(loaded) {
		    if(!default_opt.browse) {
			last_selecting=selecting;
			selecting=0;
		    }
		}
		else {
		    nc_print_files(cur_filenr);
		    nc_print_list(cur_listnr);
		}
		set_playing(playing);
		break;
	      case KEY_DC: /* Delete */
		if(!L_size(PL))
		    break;
		L_remove_nr(PL, cur_listnr);
		L_order_reset(PL);
		if(L_size(PL) && cur_listnr >= L_size(PL))
		    --cur_listnr;
		if(loaded == LOADED_LIST) {
		    if((tmp=L_order_name_linearget(PL, M.real_filename)) != -1)
			L_order_linearset(PL, tmp);
		    else
			loaded=LOADED_VOID;
		    set_playing(playing);
		}
		nc_print_list(cur_listnr);
		if(!L_size(PL)) {
		    playlist_name[0]=0;
		    goto ugly_goto;
		}
		break;
	      default: /* All other keys ignored */
	    }
	}
    }
    else { /* !selecting */
	/* The following keys are available only when we have a loaded song */
	if(loaded) {
	    switch(ch) {
	      case KEY_RIGHT:
		if(first_voice+nr_visible_voices-1 == cur_voice) {
		    first_voice=MAX(0, MIN(first_voice+1,
					   M.nr_voices-nr_visible_voices));
		    cur_voice=first_voice+nr_visible_voices-1;
		}
		else
		    cur_voice=MIN(M.nr_voices-1, cur_voice+1);
		
		nc_print_channelnumbers();
		break;
	      case KEY_LEFT:
		if(first_voice == cur_voice)
		    first_voice=cur_voice=MAX(first_voice-1, 0);
		else
		    cur_voice=MAX(0, cur_voice-1);
		
		nc_print_channelnumbers();
		break;
	      case KEY_NPAGE: /* Forward 1 pattern */
		if(playing) {
		    command_player(MESSAGE_JUMPPATTERN,
				   MIN(last_pos+1, M.songlength-1));
		    nc_clear_screen();
		}
		else {
		    last_pos=MIN(last_pos+1, M.songlength-1);
		}
		nc_print_pos(last_pos, 0);
		break;
	      case KEY_PPAGE: /* Backward 1 pattern */
		if(playing) {
		    command_player(MESSAGE_JUMPPATTERN, MAX(last_pos-1, 0));
		    nc_clear_screen();
		}
		else {
		    last_pos=MAX(last_pos-1, 0);
		}
		nc_print_pos(last_pos, 0);
		break;
	      case KEY_HOME:
		command_player(MESSAGE_JUMPPATTERN, opt.start_pos);
		reset_playingtime();
		nc_clear_screen();
		break;
	      case KEY_UP:
		first_sample=MAX(first_sample-1, 1);
		print_samples(first_sample);
		break;
	      case KEY_DOWN:
		first_sample=MAX(1, MIN(first_sample+1, M.nr_samples-7));
		print_samples(first_sample);
		break;
#if 0
	      case KEY_NPAGE:
		first_sample=MAX(1, MIN(first_sample+8, M.nr_samples-7));
		print_samples(first_sample);
		break;
	      case KEY_PPAGE:
		first_sample=MAX(first_sample-8, 1);
		print_samples(first_sample);
		break;
	      case KEY_HOME:
		first_sample=1;
		print_samples(first_sample);
		break;
	      case KEY_END:
		first_sample=MAX(1, M.nr_samples-7);
		print_samples(first_sample);
		break;
#endif
	      case '\n':
		if(playing) {
		    command_player(MESSAGE_STOP, 42);
		    nc_print_pos(last_pos, 0);
		    nc_clear_screen();
		}
		else {
		    command_player(MESSAGE_JUMPPATTERN, last_pos);
		}
		break;
	      case 'x':
		opt.active_voices=~opt.active_voices;
		goto ugly_save_space;
	      case '.':
		tmp=1<<cur_voice;
		opt.active_voices=(opt.active_voices&(~tmp))|
		    ((~(opt.active_voices&tmp))&tmp);
	      ugly_save_space:
		if(playing) {
		    command_player(MESSAGE_JUMPPATTERN, last_pos);
		    nc_clear_screen();
		}
		nc_print_channelnumbers();
		break;
	      default:
	    }
        }
	
	/* Keys available anytime we are not selecting */
	switch(ch) {
	  case ' ':
	  case 'n':
	    load_next(0, 0, 0); /* No exit after last, no persistency */
	    if(loaded == LOADED_FILE)
		cur_filenr=loaded_filenr;
	    break;
	  case 'p':
	    load_prev(0, 0, 0);  /* No exit after last, no persistency */
	    if(loaded == LOADED_FILE)
		cur_filenr=loaded_filenr;
	    break;
	  case 'l':
	    selecting=last_selecting;
	    nc_clear_screen();
	    if(!files) { /* First time */
		print_status("Loading directory...");
		init_dir();
		set_playing(playing); /* Restore status-line */
	    }
	    nc_print_selectingbar();
	    nc_print_files(cur_filenr);
	    nc_print_list(cur_listnr);
	    break;
	  case 'v':
	    if(nr_visible_voices == 4)
		nr_visible_voices=9;
	    else if(nr_visible_voices == 9)
		nr_visible_voices=17;
	    else
		nr_visible_voices=4;
	    
	    first_voice=
		MAX(0, MIN(cur_voice, M.nr_voices-nr_visible_voices));
	    
	    nc_print_channelnumbers();
	    if(playing && !default_opt.noscroll)
		nc_print_line(last_pos, last_line);
	    break;
	  default:
	}
    }
#ifdef NC_FLUSH_INPUT
    read(STDIN_FILENO, tmpname, 80); /* Flush input (for slow CPU's) */
#endif
}

static int was_playing;

void nc_suspend(void)
{
    was_playing=playing;
    if(playing) {
	command_player(MESSAGE_STOP, 42);
	nc_print_pos(last_pos, 0);
    }
    if(!selecting)
	nc_clear_screen();

}


void nc_resume(void)
{
    curs_set(0); /* Turn off cursor again as ncurses seems to miss this */
    if(was_playing)
	command_player(MESSAGE_JUMPPATTERN, last_pos);
    if(selecting) {
	nc_print_files(cur_filenr);
	nc_print_list(cur_listnr);
    }
}

#endif /* COMPILE_GUI_NC */
