/*
 *  screen.c - Handling of the screen.
 *
 *  (C) 1994, 1995 Mikael Nordqvist (d91mn@efd.lth.se, mech@df.lth.se)
 */

#include <stdio.h>

#include "mod.h"


void init_screen(void)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_init_screen();
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_init_screen();
#endif
    if(GUI_mode == GUI_SIMPLE)
	simple_init_screen();
}


void draw_screen(void)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_draw_screen();
#endif
}

void cleanup_screen(void)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_cleanup_screen();
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_cleanup_screen();
#endif
    if(GUI_mode == GUI_SIMPLE)
	simple_cleanup_screen();
}


void print_samples(int n)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
        nc_print_samples(n);
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
        tk_print_samples(n);
#endif
}


void print_songname(char *n)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_print_songname(n);
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_print_songname(n);
#endif
}


void print_filename(char *s)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_print_filename(s);
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_print_filename(s);
#endif
}


void print_status(char *s)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_print_status(s);
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_print_status(s);
#endif
}


void print_pos(int pos, int line)
{
    if(GUI_mode == GUI_SIMPLE)
	simple_print_pos(pos, line);
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_print_pos(pos, line);
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_print_pos(pos, line);
#endif
}


void print_speed(int speed, int tempo)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_print_speed(speed, tempo);
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_print_speed(speed, tempo);
#endif
}


void print_songinfo(int mode)
{
    char buf[5];
    
    switch(M.format) {
      case MODFORMAT_MOD:
	strcpy(buf, "MOD");
	break;
      case MODFORMAT_ULT:
	strcpy(buf, "ULT");
	break;
      case MODFORMAT_MTM:
	strcpy(buf, "MTM");
	break;
      case MODFORMAT_S3M:
	strcpy(buf, "S3M");
	break;
      default:
    }

#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC) {
	if(mode != -1) {
	    nc_print_songinfo(M.nr_voices, M.nr_used_samples, buf, M.size,
			      M.filesize, M.songlength, M.nr_patterns);
	    nc_print_time(0);
	}
	else {
	    nc_print_songinfo(-1, 0, 0, 0, 0, 0, 0);
	}
    }
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK) {
	if(mode != -1) {
	    tk_print_songinfo(M.nr_voices, M.nr_used_samples, buf, M.size,
			      M.filesize, M.songlength, M.nr_patterns);
	    tk_print_time(0);
	}
	else {
	    tk_print_songinfo(-1, 0, 0, 0, 0, 0, 0);
	}
    }
#endif
}    

/* -1 = empty, -2 = last value */

void print_time(int t)
{
    static int last_time=-1;
    
    if(t == -2)
	t=last_time;
    else
	last_time=t;
    
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_print_time(t);
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_print_time(t);
#endif
}


void pre_load_update_screen(void)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_pre_load_update_screen();
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_pre_load_update_screen();
#endif
    if(GUI_mode == GUI_SIMPLE)
	simple_pre_load_update_screen();
}


void post_load_update_screen(void)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_post_load_update_screen();
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_post_load_update_screen();
#endif
    if(GUI_mode == GUI_SIMPLE)
	simple_post_load_update_screen();
}


void error_load_update_screen(void)
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_error_load_update_screen();
#endif
}


void clear_all_info(void)
{
    if(!GUI_mode)
	return;

    print_samples(-1);
    print_status("");
    print_songname(" ");
    print_filename("");

    print_pos(-1,0);
    print_speed(-1,0);
    print_songinfo(-1);
    print_time(-1);
}


void print_line(int songpos, int line, int lagging)
{
    if(GUI_mode == GUI_SIMPLE)
	simple_print_line(songpos, line);
    
    if(lagging)
	return;
    
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_print_line(songpos, line);
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_print_line(songpos, line);
#endif
}


void print_playingtime(void)
{
    char tmp, day, hour, min, sec;
    
    if(!quiet && total_time && quit != QUIT_SIGNAL) {
	day=total_time/(24*3600);
	hour=(total_time/3600)%24;
	min=(total_time/60)%60;
	sec=total_time%60;
	
	printf("Total time spent playing was ");
	tmp=0;
	if(day) {
	    if(day == 1)
		printf("1 day, ");
	    else
		printf("%d days, ", day);
	    tmp=1;
	}
	if(tmp || hour) {	
	    if(hour == 1)
		printf("1 hour, ");
	    else
		printf("%d hours, ", hour);
	    tmp=1;
	}
	if(tmp || min) {
	    if(min == 1)
		printf("1 minute and ");
	    else
		printf("%d minutes and ", min);
	    tmp=1;
	}
	if(sec == 1)
	    printf("1 second.\n");
	else
	    printf("%d seconds.\n", sec);
    }
}


int print_event(char *buf, struct event *n, int vis_voices)
{
    unsigned char note=n->note;

    if(note == NOTE_OFF)
	note=12*8+BASE_NOTE; /* Last of the named notes */
    else {
	if(note && M.format != MODFORMAT_S3M)
	    note-=2*12;
    }

    if(vis_voices == 17) { /* Only notes */
	sprintf(buf, "|%s", 
		(note ? notenames[note-BASE_NOTE] : "   "));
	return 4;
    }
    else if(vis_voices == 9) { /* Notes + instruments */
	sprintf(buf, "|%s ", 
		(note ? notenames[note-BASE_NOTE] : "   "));
	
	if(n->sample)
	    sprintf(buf+5, "%2d", n->sample);
	else
	    strcpy(buf+5, "  ");
	return 7;
    }
    else { /* All of it (unless there are two effects) */
	sprintf(buf, "| %s ",
		(note ? notenames[note-BASE_NOTE] : "   "));
	
	if(n->sample)
	    sprintf(buf+6, "%2d ", n->sample);
	else
	    strcpy(buf+6, "   ");
	
	/* Check if there are one or two effects. If there are two we just
	 * display the effects without the arguments.
	 */
	if(M.format != MODFORMAT_S3M && ((!n->effect && n->arg) || n->effect)
	   && ((!n->effect2 && n->arg2) || n->effect2)) {

	    /* Two effects */
	    sprintf(buf+9, "%s+%s ", shorteffectnames[n->effect],
		    shorteffectnames[n->effect2]);
	}
	else {
	    /* Only one (or no) effect */
	    if((!n->effect && n->arg) || n->effect)
		sprintf(buf+9, "%s %02X ", effectnames[n->effect], n->arg);
	    else if(!M.format == MODFORMAT_S3M &&
		    ((!n->effect2 && n->arg2) || n->effect2))
		sprintf(buf+9, "%s %02X ", effectnames[n->effect2], n->arg2);
	    else
		strcpy(buf+9, "        ");
	}
	return 17;
    }
}


void screen_suspend()
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_suspend();
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_suspend();
#endif
}


void screen_resume()
{
#ifdef COMPILE_GUI_NC
    if(GUI_mode == GUI_NC)
	nc_resume();
#endif
#ifdef COMPILE_GUI_TK
    if(GUI_mode == GUI_TK)
	tk_resume();
#endif
}
