/*
 *  simple_screen.c - Handling of the "screen" for the simple interface.
 *
 *  (C) 1994, 1995 Mikael Nordqvist (d91mn@efd.lth.se, mech@df.lth.se)
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>

#include "mod.h"

static struct termios tio, orig_tio;

void simple_init_screen(void)
{
    /* Set up terminal (turn off linebuffering and echo) */
    tcgetattr(STDIN_FILENO, &orig_tio);
    tio=orig_tio;
    tio.c_lflag&=~(ICANON|ECHO);
    tio.c_cc[VMIN]=1;
    tio.c_cc[VTIME]=0;
    tcsetattr(STDIN_FILENO, TCSADRAIN, &tio);
}


void simple_cleanup_screen(void)
{
    tcsetattr(STDIN_FILENO, TCSADRAIN, &orig_tio);
}


void simple_print_pos(int pos, int line)
{
    char buf[160];

    if(opt.dump)
	return;
    
    sprintf(buf,"Songpos [%3d/%3d] Pattern [%3d/%3d] Line [%3d/ 63] %s",
	    pos, M.songlength-1, M.patterntable[pos], M.nr_patterns-1,
	    line, path_basename(M.real_filename, 0));
    simple_print(buf);
}


void simple_print_line(int songpos, int line)
{
    char buf[256];
    int pat, pos, v, step;

    if(!opt.dump)
	return;

    pat=M.patterntable[songpos];
    
    if(!line) {
	printf("\n\nSongpos [%3d/%3d]  Pattern [%3d/%3d]\n",
	       songpos, M.songlength-1, pat, M.nr_patterns-1);
    }
    
    sprintf(buf, "\n%02d  ", line);
    pos=5;
    for(v=0; v < M.nr_voices && v < M.nr_voices; ++v) {
	step=print_event(&buf[pos], GET_EVENT_PTR(v, pat, line), 4);
	pos+=step;
    }
    sprintf(&buf[pos], "|  %02d", line);
    pos+=5;
    printf(buf);
    fflush(stdout);
}


void simple_print(char *s)
{
    static char pre[]="\b* ";
    static char post[]="\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b"
	"\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b"
	    "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b";

    if(!s && opt.dump) {
	printf("\n");
	return;
    }

    if(s)
	printf("%s%s%s", pre, fill_string(s, 77), post);
    else
	printf("%s\b%s%s\b \b", post, fill_string("", 79), post);
    fflush(stdout);
}


void simple_pre_load_update_screen(void)
{
    if(opt.dump && !opt.verbose) {
	printf("Loading... ");
	fflush(stdout);
    }
}


void simple_post_load_update_screen(void)
{
    if(opt.dump && !opt.verbose)
	printf("Playing module \"%s\".", M.name);
}
