/*
 *  sound.c - Initializes the sounddriver.
 *
 *  (C) 1994, 1995 Mikael Nordqvist (d91mn@efd.lth.se, mech@df.lth.se)
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <limits.h>
#include <sys/soundcard.h>
#include <sys/ultrasound.h>
#include <errno.h>
#include <sys/time.h>

#include "mod.h"

/* Define a large bufffer (so there are no unwanted flushes) */

SEQ_DEFINEBUF(4096);

void init_sound(void)
{
    int nr, i, tmp;
    struct synth_info si;

    if((seqfd=open("/dev/sequencer", O_RDWR, 0)) == -1)
	error("Unable to open sequencer.\n");

    fcntl(seqfd, F_SETFL, O_NONBLOCK);

    if(ioctl(seqfd, SNDCTL_SEQ_NRSYNTHS, &nr) == -1)
	error("Unable to find out # of synth-devices.\n");

    gus_dev=-42;
    for(i=0; i < nr; ++i) {
	si.device=i;
	if(ioctl(seqfd, SNDCTL_SYNTH_INFO, &si) == -1) {
	    tmp=errno;
	    info("Unable to get synth_info for device %d/%d", i, nr);
	    if(tmp == EBUSY)
		info(" (EBUSY)");
	    else if(tmp == EBUSY)
		info(" (ENXIO)");
	    else
		info(" (errno=%d)", tmp);
	    info(".\n");
	}
	
	if(si.synth_type == SYNTH_TYPE_SAMPLE &&
	   si.synth_subtype == SAMPLE_TYPE_GUS) {
	    gus_dev=i;
	    break;
	}
    }
    
    if(gus_dev == -42)
	error("Unable to find a (non-busy) GUS on this system.\n");

    ioctl(seqfd, SNDCTL_SEQ_RESET, 0);
    
}

void cleanup_sound(void) 
{
    close (seqfd);
}
