/*
 *  tk_controller.c - Controller for the Tk interface.
 *
 *  (C) 1994, 1995 Mikael Nordqvist (d91mn@efd.lth.se, mech@df.lth.se)
 */

#ifdef COMPILE_GUI_TK

#include <stdlib.h>
#include <time.h>
#include <errno.h>
#include <string.h>
#include <tk.h>
#include <unistd.h>
#include <fcntl.h>
#include <limits.h>

#include "mod.h"
#include "message.h"

static int tk_ready=0; /* Makes sure we don't call c_quit before it exists */

/*
 * Functions to set up the TK interface.
 */


void tk_parse_options(int *argc, char *argv[])
{
    Et_Init(argc, argv, TK_ARGV_NO_ABBREV);
}

static void tk_seqfd_callback(ClientData cd, int mask)
{
    read_sequencer();
}

void tk_call_mainloop(void)
{
    Tk_CreateFileHandler(seqfd, TK_READABLE, tk_seqfd_callback,
			 (ClientData)42);
    if(quit)
	tk_quit_signal();
    Et_MainLoop();
    exit(0);        /* Never reached */
}

void tk_init_controller(void)
{
    ET(change_songpos 0
       $w_samplelistbox yview 0
       update);
    tk_ready=1;

    cleanup_sound(); /* Make sure everything is hunky-dory after TK has */
    init_sound();    /* done it's dirty deed. */
    
    L_order_reset(PL);
    if(default_opt.auto_next && L_size(PL)) {
	load_next(0, 0, 0); /* No exit after last, no persistency */
    }
    else {
	ET($w_load invoke); /* Bring up load requester */
    }
    tk_dim_buttons();
}

void tk_end_of_song(void)
{
    if(!selecting && default_opt.auto_next && loaded == LOADED_LIST)
	load_next(0, 0, 0); /* No exit after last, no persistency */
    tk_dim_buttons();
}

void tk_quit_signal(void)
{
    quit=QUIT_SIGNAL;
    if(tk_ready)
	ET(after 1 c_quit);
    else
	exit(1);
}

/* Volume stuff */

ET_PROC(c_sync_volume)
{
    sync_volume();
    ET($w_volscale set "%d(get_mastervolume())"
       set mute_state %d(is_mute()));
    
    return ET_OK;
}

ET_PROC(c_setmute)
{
    set_mute(ET_INT(set mute_state));
    ET(set mute_state "%d(is_mute())");
    return ET_OK;
}

ET_PROC(c_volume)
{
    set_mastervolume(atoi(argv[1]));
    return ET_OK;
}

/*
 *  Main controller window.
 */

ET_PROC(c_quit)
{
    if(loaded)
	command_player(MESSAGE_EXIT, 42);
    cleanup();
    ET(exit);
    return ET_OK; /* Remove warning */
}

ET_PROC(c_next)
{
    ET(update idletasks);
    load_next(0, 0, 0); /* No exit after last, no persistency */
    tk_dim_buttons();
    return ET_OK;
}

ET_PROC(c_prev)
{
    ET(update idletasks);
    load_prev(0, 0, 0); /* No exit after last, no persistency */
    tk_dim_buttons();
    return ET_OK;
}

ET_PROC(c_restart)
{
    if(loaded) {
	tk_print_pos(opt.start_pos, 0);
	ET(change_songpos "%d(opt.start_pos)");
	if(!playing)
	    command_player(MESSAGE_JUMPPATTERN, last_pos);
	reset_playingtime();
    }

    return ET_OK;
}

ET_PROC(c_changesongpos)
{
    int pos;
    if(loaded) {
	pos=atoi(argv[1]);
	if(playing)
	    command_player(MESSAGE_JUMPPATTERN, pos);
	else {
	    last_pos=pos;
	    tk_print_pos(last_pos, 0);
	}
    }
    return ET_OK;
}

ET_PROC(c_playstop)
{
    if(loaded) {
	if(playing) {
	    command_player(MESSAGE_STOP, 42);
	    tk_print_pos(last_pos, 0);
	}
	else {
	    command_player(MESSAGE_JUMPPATTERN, last_pos);
	}
    }
    tk_dim_buttons();
    return ET_OK;
}

ET_PROC(c_toggle_browse)
{
    if(atoi(argv[1]))
	default_opt.browse=1;
    else
	default_opt.browse=0;
    return ET_OK;
}

ET_PROC(c_toggle_random)
{
    int nr;

    if(atoi(argv[1]))
	default_opt.random_mode=1;
    else
	default_opt.random_mode=0;

    L_order_reset(PL);
    if(loaded == LOADED_LIST) {
	nr=L_order_name_linearget(PL, M.real_filename);
	if(default_opt.random_mode)
	    L_order_linear_swapfirst(PL, nr);
	if(nr == -1)
	    debug("Something wrong here...\n");
	L_order_linearset(PL, nr);
    }

    tk_dim_buttons();
    return ET_OK;
}

/*
 *  Load mode.
 */

ET_PROC(c_load)
{
    if(default_opt.allow_delete)
	ET(.l.delete configure -state normal);

    if(default_opt.browse)
	ET(set browse_state 1);
    else
	ET(set browse_state 0);

    if(default_opt.random_mode)
	ET(set random_state 1);
    else
	ET(set random_state 0);

    selecting=1;
    if(!files) /* First time */
	tk_init_dir();
    tk_show_dir();
    tk_show_list();

    return ET_OK;
}


void tk_dim_buttons(void)
{
    char prev=0, next=0;
    
    if(loaded == LOADED_FILE) {
	if(loaded_filenr > nr_dirs)
	    prev=1;
	if(loaded_filenr < nr_files-1)
	    next=1;
    }
    else if(loaded == LOADED_LIST) {
	if(L_size(PL)) {
	    if(default_opt.replay_forever || !L_isatstart(PL))
		prev=1;
	    if(default_opt.replay_forever || L_isreset(PL) || !L_isatend(PL))
		next=1;
	}
    }
    else {
	if(L_size(PL)){
	    if(L_isreset(PL)) {
		next=1;
	    }
	    else {
		if(!L_isatstart(PL))
		    prev=1;
		if(!L_isatend(PL))
		    next=1;
	    }
	}
    }
    
    if(next)
	ET(foreach a $ww_next {$a configure -state normal}
	   $w_menu_file entryconfigure $w_menu_file_nr_next -state normal);
    else
	ET(foreach a $ww_next {$a configure -state disabled}
	   $w_menu_file entryconfigure $w_menu_file_nr_next -state disabled);
    
    if(prev)
	ET(foreach a $ww_prev {$a configure -state normal}
	   $w_menu_file entryconfigure $w_menu_file_nr_prev -state normal);
    else
	ET(foreach a $ww_prev {$a configure -state disabled}
	   $w_menu_file entryconfigure $w_menu_file_nr_prev -state disabled);


    if(loaded) {
	if(playing)
	    ET(foreach a $ww_playstop {$a configure -bitmap stop});
	else
	    ET(foreach a $ww_playstop {$a configure -bitmap play});
	
	ET(foreach a $ww_playstop {$a configure -state normal});
    }
    else
	ET(foreach a $ww_playstop {$a configure -bitmap play}
	   foreach a $ww_playstop {$a configure -state disabled});

    /* Restore pointer */
    ET(.m configure -cursor top_left_arrow);
    if(ET_INT(expr {$load_active}))
	ET(.l configure -cursor top_left_arrow);

}


void tk_init_dir(void)
{
    ET(.l.filebox delete 0 end
       set m_load_dir "Loading directory..."
       update idletasks
       );
    init_dir();
}

void tk_show_dir(void)
{
    int i;

    ET(set m_load_dir "%s(escape_name(trunc_filename(workdir, 25)))");
    
    for(i=0; i < nr_files; ++i) {
	if(is_dir(i))
	    ET(.l.filebox insert end "%s(escape_name(files[i].name))/");
	else
	    ET(.l.filebox insert end "%s(escape_name(files[i].name))");
    }
    ET(set nr_files "%d(nr_files)"
       set nr_dirs "%d(nr_dirs)");
}

ET_PROC(c_unshowload)
{
    selecting=0;
    return ET_OK;
}

/* Load a file not on the playlist/change directory */

ET_PROC(c_loadfile)
{
    char name[PATH_MAX+1];
    int nr;

    nr=atoi(argv[1]);
    if(!is_dir(nr)) {
	cur_filenr=nr;
	ET(if {[expr $browse_state == 0]} {
	    .l.done invoke
	    update
	    update idletasks
	});
	strcpy(name, workdir);
	strcat(name, "/");
	strcat(name, get_filename(nr));
	if(!change_module(name, "")) {
	    loaded=0;
	    ET($w_load invoke);
	}
	else {
	    loaded=LOADED_FILE;
	    loaded_filenr=nr;
	}
	set_playing(playing);
    }
    else
	tk_change_dir(get_filename(nr));

    tk_dim_buttons();
    return ET_OK;
}

ET_PROC(c_change_dir)
{
    tk_change_dir(expand_name(argv[1]));
    return ET_OK;
}

void tk_change_dir(char *dirname)
{
    if(change_dir(dirname)) {
	tk_init_dir();
	tk_show_dir();
	cur_filenr=nr_dirs;
	if(nr_files == nr_dirs)
	    cur_filenr=-1;
	set_playing(playing); /* Restore status-line */
    }
    else {
	ET(show_message "Message" "Unable to load directory." 200);
    }
    tk_dim_buttons();
}

/*
 * Handling of the playlist
 */

void tk_show_list(void)
{
    int i;

    ET(.l.listbox delete 0 end);
    L_iterate_reset(PL);
    for(i=0; i < L_size(PL); ++i) {
	ET(.l.listbox insert end "%s(escape_name(path_basename(
           PL->current_iter->filename, default_opt.dir_components)))");
	L_iterate_forward(PL);
    }
    ET(set nr_listfiles "%d(L_size(PL))");
}

ET_PROC(c_loadlistfile)
{
    int nr;
    
    if(L_size(PL)) {
	ET(if {[expr $browse_state == 0]} {
	    .l.done invoke
	    update
	    update idletasks
	});
	nr=atoi(argv[1]);
	if(default_opt.random_mode) {
	    L_order_reset(PL);
	    L_order_linear_swapfirst(PL, nr);
	    if(loaded==LOADED_FILE)
		loaded=LOADED_VOID; /* Make sure load_next() starts the list */
	    ET(c_next);
	}
	else {
	    L_order_linearset(PL, nr);
	    if(!change_module(PL->current->filename, PL->current->options)) {
		loaded=0;
		ET($w_load invoke);
	    }
	    else
		loaded=LOADED_LIST;
	    
	    tk_dim_buttons();
	    set_playing(playing);
	}
    }
    return ET_OK;
}

/* Args: 1) insert_after 2) number to insert before/after */

ET_PROC(c_listinsertnr)
{
    char name[PATH_MAX+1], *s, *s_new;
    int nr, curnr, after, topnr;

    after=atoi(argv[1]);
    curnr=atoi(argv[2]);
    s=argv[3];
    topnr=ET_INT(expr [lindex [.l.listscroll get] 2]);

    if(!L_size(PL))
	curnr=0;

    /* Only change topnr if we insert somewhere not visible */
    if(!(after && curnr >= topnr && curnr < topnr+14) &&
       !(!after && curnr > topnr && curnr <= topnr+14))
	topnr=curnr+(after ? 1 : 0);

    if(!*s) { /* No marked file, take currently loaded song */
	if(!loaded || loaded == LOADED_LIST)
	    return ET_OK;
	if(after) {
	    L_insert_after_nr(PL, curnr, M.real_filename, "");
	    if(L_size(PL) > 1)
		curnr++;
	}
	else
	    L_insert_before_nr(PL, curnr, M.real_filename, "");
    }
    else {
	/* Skip over the directories and insert the first vaild file */
	while(*s) {
	    nr=strtol(s, &s_new, 10);
	    s=s_new;
	    if(!is_dir(nr)) {
		strcpy(name, workdir);
		strcat(name, "/");
		strcat(name, get_filename(nr));
		if(after) {
		    L_insert_after_nr(PL, curnr, name, "");
		    if(L_size(PL) > 1)
			curnr++;
		}
		else
		    L_insert_before_nr(PL, curnr, name, "");
		break;
	    }
	}
	/* Now do the rest */
	while(*s) {
	    nr=strtol(s, &s_new, 10);
	    s=s_new;
	    if(!is_dir(nr)) {
		strcpy(name, workdir);
		strcat(name, "/");
		strcat(name, get_filename(nr));
		L_insert_after_nr(PL, curnr++, name, "");
	    }
	}
    }

    tk_show_list();
    ET(change_listpos "%d(topnr)");
    ET(.l.filebox selection clear 0 end);

    L_order_reset(PL);
    if(loaded == LOADED_LIST) {
	nr=L_order_name_linearget(PL, M.real_filename);
	if(nr == -1)
	    debug("Something wrong here...\n");
	L_order_linearset(PL, nr);
    }

    tk_dim_buttons();
    return ET_OK;
}

/* The list of files to be removed MUST be sorted in increasing order */

ET_PROC(c_removelistfiles)
{
    int nr, removed, topnr;
    char *s, *s_new;

    topnr=ET_INT(expr [lindex [.l.listscroll get] 2]);
    s=argv[1];
    removed=0;
    if(!*s) { /* No marked file, take currently loaded song */
	if(loaded != LOADED_LIST)
	    return ET_OK;
	nr=L_order_name_linearget(PL, M.real_filename);
	if(nr == -1)
	    debug("Something wrong here...\n");
	L_remove_nr(PL, nr-removed++);
    }
    else {
	while(*s) {
	    nr=strtol(s, &s_new, 10);
	    if(nr == topnr)
		topnr=0;
	    s=s_new;
	    L_remove_nr(PL, nr-removed++);
	}
    }
    tk_show_list();
    ET(change_listpos "%d(topnr)");
    ET(.l.listbox selection clear 0 end);

    L_order_reset(PL);
    if(loaded == LOADED_LIST) {
	if((nr=L_order_name_linearget(PL, M.real_filename)) != -1)
	    L_order_linearset(PL, nr);
	else
	    loaded=LOADED_VOID;
    }
    set_playing(playing);
    
    if(!L_size(PL)) {
	if(PL->list_name) {
	    free(PL->list_name);
	    PL->list_name=0;
	}
    }
    tk_dim_buttons();
    return ET_OK;
}


ET_PROC(c_deletefile)
{
    char buf[PATH_MAX+1];
    int nr, topnr;

    if(!default_opt.allow_delete) /* Safe-guard */
	return ET_OK;
    
    nr=atoi(argv[1]);
    topnr=ET_INT(expr [lindex [.l.filescroll get] 2]);
    if(!is_dir(nr)) {
	strcpy(buf, workdir);
	strcat(buf, "/");
	strcat(buf, get_filename(nr));
	if(!unlink(buf)) { /* Delete successful */
	    remove_file_nr(nr);
	    if(loaded == LOADED_FILE)
		loaded=LOADED_VOID;
	    ET(.l.filebox delete "%d(nr)"
	       set nr_files [expr $nr_files-1]
	       change_filepos "%d(topnr)");
	}
	ET(.l.filebox selection clear 0 end);
    }
    tk_dim_buttons();
    return ET_OK;
}


/* Arguments 1: 0 = name, 1 = #, 2: name/# */

ET_PROC(c_load_playlist)
{
    char buf[PATH_MAX+1];
    
    if(atoi(argv[1])) {
	strcpy(buf, workdir);
	strcat(buf, "/");
	strcat(buf, get_filename(atoi(argv[2])));
    }
    else
	strcpy(buf, expand_name(argv[2]));
    
    if(!load_playlist(buf, PL)) {
	ET(set name "%s(buf)"
	   show_message "Loading failed..." "Error reading list from

'$name'" 300);
    }
    else {
	L_order_reset(PL);
	tk_show_list();
	if(loaded == LOADED_LIST)
	    loaded=LOADED_VOID;
	set_playing(playing);
    }
    
    tk_dim_buttons();
    return ET_OK;
}

ET_PROC(c_save_playlist)
{
    if(!save_playlist(expand_name(argv[1]), PL)) {
	ET(set name "%s(argv[1])"
	   show_message "Saving failed..." "Error saving list to

'$name'" 300);
    }
    else {
	tk_init_dir();
	tk_show_dir();
    }

    return ET_OK;
}

ET_PROC(c_setcwd)
{
    ET(set c2tcl "%s(workdir)");
    return ET_OK;
}

ET_PROC(c_setlistname)
{
    if(PL->list_name)
	ET(set c2tcl "%s(PL->list_name)");
    else
	ET(set c2tcl "");
    return ET_OK;
}


ET_PROC(c_access)
{
    if(!access(argv[1], R_OK))
	ET(set c2tcl 1);
    else
	ET(set c2tcl 0);
    return ET_OK;
}


ET_PROC(c_viewcomment)
{
    char buf[]=TMP_DIR "/module_comment";
    int fd;
    
    fd=open(buf, O_CREAT|O_TRUNC|O_RDWR, 0777);
    write(fd, M.songtext, strlen(M.songtext));
    
    ET(set tmp [open "%s(buf)" "r"]
       view_text "Module comment" [read $tmp]
       close $tmp);
    unlink(buf);
    return ET_OK;
}


ET_PROC(c_viewinfofile)
{
    char buf[80]=TMP_DIR "/module_infofile";
    int fd;
    
    fd=open(buf, O_CREAT|O_TRUNC|O_RDWR, 0777);
    write(fd, M.infofile, strlen(M.infofile));
    
    ET(set tmp [open "%s(buf)" "r"]
       view_text "Infofile from archive" [read $tmp]
       close $tmp);
    unlink(buf);
    return ET_OK;
}

int tk_get_entry_int(char *entry)
{
    char buf[80];
    char *p, *p2;
    int val;

    sprintf(buf, "%s get", entry);
    p=ET_STR(set foo [%s(buf)]);
    val=strtol(p, &p2, 10);
    if(*p && p2 && p != p2 && val >= 0)
	return val;
    return -1;
}    

/*
 * Functions to set the optionswindow(s) according to current options.
 */

ET_PROC(c_set_globaloptions)
{
    tk_set_options(1, &default_opt);

    return ET_OK;
}

ET_PROC(c_set_moduleoptions)
{
    struct options o;
    int nr;
    
    nr=atoi(argv[1]);
    
    ET(set m_moduleoptions_name "%s(escape_name(path_basename(L_linear_get_filename_nr(PL, nr), 0)))");
    debug("<%s>\n", L_get_options_nr(PL, nr));
    reset_options(&o);
    process_options_str(L_get_options_nr(PL, nr), &o);
    tk_set_options(0, &o);

    return ET_OK;
}

void tk_set_options(int global, struct options *o)
{
    if(global) {
	if(o->auto_next)
	    ET(.o.oA deselect);
	else
	    ET(.o.oA select);

	if(o->browse)
	    ET(.o.ob select);
	else
	    ET(.o.ob deselect);
	
	if(o->replay_forever)
	    ET(.o.oC select);
	else
	    ET(.o.oC deselect);
	
	if(o->random_mode)
	    ET(.o.or select);
	else
	    ET(.o.or deselect);
	
	if(o->allow_delete)
	    ET(.o.oZ select);
	else
	    ET(.o.oZ deselect);
	
	ET(.o.oX delete 0 end);
	if(o->dir_components > 0)
	    ET(.o.oX insert 0 "%d(o->dir_components)");
	
	ET(.o.oD delete 0 end);
	if(strcmp(startup_dir, "."))
	    ET(.o.oD insert 0 "%s(startup_dir)");

	ET(.o.oY delete 0 end);
	if(strcmp(mixer_name, DEFAULT_MIXER))
	    ET(.o.oY insert 0 "%s(mixer_name)");
	
	ET(.o.oI%d(GUI_size) invoke);
    }
    
    /* Module specific */

    if(o->break_loops)
	ET(.o.oB deselect);
    else
	ET(.o.oB select);
    
    if(o->loop_module)
	ET(.o.oL select);
    else
	ET(.o.oL deselect);

    if(o->mono)
	ET(.o.oM select);
    else
	ET(.o.oM deselect);

    if(o->tolerant)
	ET(.o.oT select);
    else
	ET(.o.oT deselect);

    if(o->speed0stop)
	ET(.o.o0 deselect);
    else
	ET(.o.o0 select);

    if(!o->high_note)
	ET(.o.ooO invoke);
    else if(o->high_note-o->low_note == 3*12-1)
	ET(.o.oo invoke);
    else
	ET(.o.oO invoke);

    ET(.o.fl.el.ot delete 0 end;
       .o.fl.el.ot configure -state disabled);
    if(o->nobpm && o->tempo == 125)
	ET(.o.o5 invoke);
    else if(o->nobpm && o->tempo == 150)
	ET(.o.o6 invoke);
    else  {
	ET(.o.o56 invoke;
	   .o.fl.el.ot configure -state normal);
	if(o->tempo != 125)
	    ET(.o.fl.el.ot insert 0 "%d(o->tempo)");
    }
    
    if(o->ntsc_samples)
	ET(.o.oN invoke);
    else
	ET(.o.oP invoke);
    
    ET(.o.fl.el.os delete 0 end);
    if(o->speed != 6)
	ET(.o.fl.el.os insert 0 "%d(o->speed)");

    ET(.o.fl.el.op delete 0 end);
    if(o->start_pos)
	ET(.o.fl.el.op insert 0 "%d(o->start_pos)");
    
    ET(.o.fl.el.oS delete 0 end);
    if(o->stop_songpos || o->stop_linepos)
	ET(.o.fl.el.oS insert 0 "%d(o->stop_songpos),%d(o->stop_linepos)");

    ET(.o.fl.el.oc delete 0 end);
    if(o->click_removal)
	ET(.o.fl.el.oc insert 0 "%d(o->click_removal)");

    /* -F */

    ET(.o.fl.el.om delete 0 end);
    if(o->maxtime)
	ET(.o.fl.el.om insert 0 "%d(o->maxtime)");

    if(!global) {
	switch(o->format) {
	  case MODFORMAT_MOD:
	    ET(.o.ofmod invoke);
	    break;
	  case MODFORMAT_MTM:
	    ET(.o.ofmtm invoke);
	    break;
	  case MODFORMAT_ULT:
	    ET(.o.ofult invoke);
	    break;
	  case MODFORMAT_S3M:
	    ET(.o.ofs3m invoke);
	    break;
	  default:
	    ET(.o.ofauto invoke);
	    break;
	}
    }
}

/*
 * Functions to retrieve the options from the window and use/save them.
 */

ET_PROC(c_use_globaloptions)
{
    tk_accept_options(0);
    return ET_OK;
}


ET_PROC(c_saveanduse_globaloptions)
{ 
    tk_accept_options(1);
    return ET_OK;
}

ET_PROC(c_use_moduleoptions)
{
    char buf[256];
    
    tk_get_options(buf, 0);
    L_set_options_nr(PL, atoi(argv[1]), buf);
    debug("New options : '%s'\n", buf);
    return ET_OK;
}

void tk_accept_options(char save)
{
    struct options o;
    char buf[256];
    char restart=0;
    int nr;
    
    o=opt;
    tk_get_options(buf, 1);
    if(!strstr("-I", buf))
	GUI_size=3;            /* Reset window size of no -I present */
    if(save)
	save_rcfile(buf);
    reinit_options(buf);

    if(playing && (opt.nobpm != o.nobpm || opt.speed0stop != o.speed0stop ||
		   opt.mono != o.mono || opt.break_loops != o.break_loops ||
		   opt.loop_module != o.loop_module ||
		   opt.stop_songpos != o.stop_songpos ||
		   opt.stop_linepos != o.stop_linepos))
	restart=1;
    
    if(loaded) {
	if(restart) {
	    command_player(MESSAGE_STOP, 42);
	    command_player(MESSAGE_UPDATEOPTIONS, 0);
	    command_player(MESSAGE_JUMPPATTERN, last_pos);
	}
	else
	    command_player(MESSAGE_UPDATEOPTIONS, 0);
    }

    if(default_opt.random_mode != o.random_mode) {
	L_order_reset(PL);
	if(loaded == LOADED_LIST) {
	    nr=L_order_name_linearget(PL, M.real_filename);
	    if(nr == -1)
		debug("Something wrong here...\n");
	    L_order_linearset(PL, nr);
	}
    }
    if(ET_INT(expr {$load_active})) {
	if(default_opt.dir_components != o.dir_components)
	    tk_show_list();
	if(default_opt.allow_delete != o.allow_delete) {
	    if(default_opt.allow_delete)
		ET(.l.delete configure -state normal);
	    else
		ET(.l.delete configure -state disabled);
	}
	    
    }

    if(opt.browse)
	ET(set browse_state 1);
    else
	ET(set browse_state 0);

    if(opt.random_mode)
	ET(set random_state 1);
    else
	ET(set random_state 0);

    tk_dim_buttons();
}


void tk_get_options(char *buf, char global)
{
    char *p, *p2, tmpbuf[80];
    int tmp, v1, v2;

    *buf=0;
    
    if(global) {
	strcpy(buf, ET_STR(concat $o_A $o_b $o_C $o_r $o_Z $o_I));
	if(*buf)
	    strcat(buf, " ");
    }
    
    strcat(buf, ET_STR(concat $o_B $o_L $o_M $o_NP $o_oO $o_56 $o_oT $o_o0));

    if(!global) {
	strcpy(tmpbuf, ET_STR(concat $o_of));
	if(tmpbuf[0]) {
	    strcat(buf, " ");
	    strcat(buf, tmpbuf);
	}
    }
   
    if(global) {
	p=ET_STR(.o.oD get);
	if(*p && strcmp(p, ".")) {
	    strcat(buf, " -D");
	    strcat(buf, escape_name_tilde(p));
	    strcpy(startup_dir, p);
	}
	else if(!*p || !strcmp(p, "."))
	    strcpy(startup_dir, ".");
	
	p=ET_STR(.o.oY get);
	if(*p && strcmp(p, DEFAULT_MIXER)) {	
	    strcat(buf, " -Y");
	    strcat(buf, p);
	    strcpy(mixer_namebuf, p);
	    mixer_name=mixer_namebuf;
	}
	else
	    mixer_name=DEFAULT_MIXER;
	
	tmp=tk_get_entry_int(".o.oX");
	if(tmp > 0)
	    sprintf(&buf[strlen(buf)], " -X%d", tmp);
    }
    
    tmp=tk_get_entry_int(".o.fl.el.os");
    if(tmp > 0 && tmp != 6)
	sprintf(&buf[strlen(buf)], " -s%d", tmp);

    if(!strstr(buf, "-5") && !strstr(buf, "-6")) {
	tmp=tk_get_entry_int(".o.fl.el.ot");
	if(tmp > 0 && tmp != 125)
	    sprintf(&buf[strlen(buf)], " -t%d", tmp);
    }
    
    tmp=tk_get_entry_int(".o.fl.el.op");
    if(tmp > 0)
	sprintf(&buf[strlen(buf)], " -p%d", tmp);
    
    p=ET_STR(.o.fl.el.oS get);
    v1=strtol(p, &p2, 10);
    if(*p && p2 && p != p2 && v1 >= 0 && *p2 == ',') {
	p=++p2;
	v2=strtol(p, &p2, 10);
	if(*p && p2 && p != p2 && v2 >= 0 && v2 <= 63)
	    sprintf(&buf[strlen(buf)], " -S%d,%d", v1, v2);
    }
    
    tmp=tk_get_entry_int(".o.fl.el.oc");
    if(tmp > 0)
	sprintf(&buf[strlen(buf)], " -c%d", tmp);
    
    tmp=tk_get_entry_int(".o.fl.el.om");
    if(tmp > 0)
	sprintf(&buf[strlen(buf)], " -m%d", tmp);

    while(*buf == ' ') /* Should only be one, but... */
	memmove(&buf[0], &buf[1], strlen(buf));

    /* Reset mixer in case it was changed */
    cleanup_mixer();
    init_mixer();
    ET(c_sync_volume);
}


void tk_suspend(void)
{
}


void tk_resume(void)
{
}

#endif /* COMPILE_GUI_TK */
