
######### Options screen #########

proc edit_global_options {} {
    global options_active w_menu_settings w_menu_settings_nr_options
    
    # Make sure a second click is ignored.
    if {[expr $options_active]} {
	return
    } else {
	set options_active 1
  	$w_menu_settings entryconfigure $w_menu_settings_nr_options\
		-state disabled
    }

    toplevel .o
    wm title .o "Global options"
    wm protocol .o WM_DELETE_WINDOW {.o.cancel invoke}
    wm resizable .o no no
    grab .o

    frame .o.f0 -bd 2 -relief raised
    frame .o.f
    frame .o.f.c1
    frame .o.f.c2
    
    mk_globaloption_buttons
    mk_moduleoption_buttons
    label .o.l1 -text "General options:" -relief raised -bd 1
    label .o.l2 -text "Module specific options:" -relief raised -bd 1
    label .o.l3 -text "Module specific options:" -relief raised -bd 1

    # Global
    pack .o.l1 -in .o.f.c1 -fill both
    frame .o.f.c1a -bd 1 -relief raised
    pack .o.oA .o.ob .o.oC .o.or .o.oZ -in .o.f.c1a -anchor w -expand yes\
	    -fill y
    pack .o.oXf .o.oYf -in .o.f.c1a -expand yes -fill x -pady 2
    pack .o.oDl -anchor w -in .o.f.c1a -padx 5 -expand yes -fill y
    pack .o.oD -fill x -anchor w -in .o.f.c1a -padx 5 -pady 2 -expand yes
    pack .o.oIf -fill x -in .o.f.c1a -expand yes

    # Module specific - col 1
    frame .o.f.c1b -bd 1 -relief raised
    pack .o.oB .o.oL .o.oM .o.oT .o.o0\
	    -in .o.f.c1b -anchor w -expand yes -fill y
    pack .o.f.c1a .o.l2 .o.f.c1b -in .o.f.c1 -fill x -expand yes -fill both
    
    # Module specific - col 2
    pack .o.l3 -in .o.f.c2 -fill both
    pack .o.fxo .o.fx56 .o.fxNP\
	    -in .o.f.c2 -anchor w -fill both -expand yes
    pack .o.fl -in .o.f.c2 -expand yes -fill y

    c_set_globaloptions
    
    # The rest
    pack .o.f.c1 -side left -expand yes -fill y
    pack .o.f.c2 -expand yes -fill y

    frame .o.f2 -bd 1 -relief raised
    button .o.use -text "Use" -command {
	c_use_globaloptions
	edit_global_options_cleanup
    }
    button .o.save -text "Save & Use" -command {
	c_saveanduse_globaloptions
	edit_global_options_cleanup
    }
    button .o.cancel -text "Cancel" -command {
	edit_global_options_cleanup
    }
    pack .o.use .o.save .o.cancel -padx 3 -pady 3 -fill x -expand yes\
	    -in .o.f2 -side left
    pack .o.f .o.f2 -fill both -expand yes -in .o.f0
    pack .o.f0

    bind .o <Escape> {.o.cancel invoke}
#    bind .o <Enter> {focus .o}
    focus .o
    tkwait window .o
}

proc edit_global_options_cleanup {} {
    global options_active w_menu_settings w_menu_settings_nr_options

    destroy .o
    set options_active 0
    $w_menu_settings entryconfigure $w_menu_settings_nr_options\
	    -state normal
}

proc edit_module_options {edit_list} {
    global options_active e_nr m_moduleoptions_name\
	    w_menu_settings w_menu_settings_nr_options
    
    # Make sure a second click is ignored.
    if {[expr $options_active]} {
	return
    }
    set options_active 1

    toplevel .o
    wm title .o "Module options"
    wm protocol .o WM_DELETE_WINDOW {.o.cancel invoke}
    wm resizable .o no no
    grab .o

    frame .o.f0 -bd 2 -relief raised
    frame .o.f
    frame .o.f.c1
    frame .o.f.c2
    
    mk_moduleoption_buttons

    # Module specific - col 1
    pack .o.fxf .o.fl -in .o.f.c1 -fill both -expand yes

    # Module specific - col 2

    frame .o.f.c2a -bd 1 -relief raised
    pack .o.oB .o.oL .o.oM .o.oT .o.o0\
	    -in .o.f.c2a -anchor w -expand yes

    pack .o.fxo .o.fx56 .o.fxNP .o.f.c2a\
	    -in .o.f.c2 -anchor w -fill x

    # Use/skip/cancel
    pack .o.f.c1 -side left -fill y
    pack .o.f.c2

    label .o.ofl -textvariable m_moduleoptions_name -relief raised -anchor c
    frame .o.f2 -bd 1 -relief raised
    button .o.use -text "Use" -command {
	global e_nr
	c_use_moduleoptions $e_nr
	set e_nr 0
    }
    button .o.skip -text "Skip" -command {
	global e_nr
	set e_nr 0
    }
    button .o.cancel -text "Cancel" -command {
	global e_nr
	set e_nr 1
    }
    pack .o.use .o.skip .o.cancel -padx 3 -pady 3 -fill x -expand yes\
	    -in .o.f2 -side left

    if {[expr [llength $edit_list] == 0]} {
	pack forget .o.skip
    }
    pack .o.ofl -in .o.f0 -fill x
    pack .o.f .o.f2 -fill x -in .o.f0
    pack .o.f0

    bind .o <Escape> {.o.cancel invoke}
#    bind .o <Enter> {focus .o}
    focus .o

    # Edit the modules one by one.
    if {[expr [llength $edit_list] == 0]} {
	tkwait variable e_nr
    } else {
	foreach e_nr $edit_list {
	    c_set_moduleoptions $e_nr
	    tkwait variable e_nr
	    if {[expr $e_nr]} {
		break
	    }
	}
    }
    destroy .o
    set options_active 0
    $w_menu_settings entryconfigure $w_menu_settings_nr_options\
	    -state normal
}

proc mk_globaloption_buttons {} {
    checkbutton .o.oA -variable o_A -onvalue "-A" -offvalue " "\
	    -text "Don't autoload songs"
    checkbutton .o.ob -variable o_b -onvalue "-b" -offvalue " "\
	    -text "Browse mode"
    checkbutton .o.oC -variable o_C -onvalue "-C" -offvalue " "\
	    -text "Cyclic play of modules"
    checkbutton .o.or -variable o_r -onvalue "-r" -offvalue " "\
	    -text "Randomized playorder"
    checkbutton .o.oZ -variable o_Z -onvalue "-Z" -offvalue " "\
	    -text "Allow deletion of files"

    frame .o.oXf
    label .o.oXl -text "Path components"
    frame .o.oXfx1
    entry .o.oX -relief sunken -width 8
    bind .o.oX <Return> {focus .o}
    pack .o.oXl -padx 4 -side left -in .o.oXf
    pack .o.oXfx1 -side left -fill x -expand yes -in .o.oXf
    pack .o.oX -in .o.oXf -padx 5

    label .o.oDl -text "Startup directory"
    entry .o.oD -relief sunken
    bind .o.oD <Return> {focus .o}

    frame .o.oYf
    label .o.oYl -text "Mixer"
    entry .o.oY -relief sunken
    pack .o.oYl -in .o.oYf -padx 4 -side left
    pack .o.oY -in .o.oYf -padx 5 -side left -fill x -expand yes
    bind .o.oY <Return> {focus .o}

    frame .o.oIf
    frame .o.oIfx
    frame .o.oIfxl
    frame .o.oIfxr
    label .o.oIl -text "Window size"
    radiobutton .o.oI1 -variable o_I -value "-Ichubby" -text "Chubby"
    radiobutton .o.oI2 -variable o_I -value "-Islim" -text "Slim"
    radiobutton .o.oI3 -variable o_I -value " " -text "Normal"
    radiobutton .o.oI4 -variable o_I -value "-Iverbose" -text "Verbose"
    pack .o.oI3 .o.oI1 -in .o.oIfxl -anchor w
    pack .o.oI4 .o.oI2 -in .o.oIfxr -anchor w
    pack .o.oIfxl .o.oIfxr -in .o.oIfx -side left -expand yes
    pack .o.oIl .o.oIfx -in .o.oIf -expand yes -fill x
}
    
proc mk_moduleoption_buttons {} {
    checkbutton .o.oB -variable o_B -onvalue "-B" -offvalue " "\
	    -text "Allow infinite loops"
    checkbutton .o.oL -variable o_L -onvalue "-L" -offvalue " "\
	    -text "Loop module"
    checkbutton .o.oM -variable o_M -onvalue "-M" -offvalue " "\
	    -text "Mono output"

    frame .o.fxf -bd 1 -relief raised 
    label .o.lxf -text "Module format"
    frame .o.fxfx
    frame .o.fxfx2

    radiobutton .o.ofauto -variable o_of -value " "    -text "Auto detect"
    radiobutton .o.ofmod -variable o_of -value "-fmod" -text "MOD"
    radiobutton .o.ofmtm -variable o_of -value "-fmtm" -text "MTM"
    radiobutton .o.ofult -variable o_of -value "-fult" -text "ULT"
    radiobutton .o.ofs3m -variable o_of -value "-fs3m" -text "S3M"

    pack .o.ofauto .o.ofmod -side left -in .o.fxfx -fill x -expand yes
    pack .o.ofmtm .o.ofult .o.ofs3m -side left -in .o.fxfx2 -fill x -expand yes
    pack .o.lxf .o.fxfx .o.fxfx2 -in .o.fxf
 
    frame .o.fxNP -bd 1 -relief raised
    label .o.lxNP -text "Samplerate" 
    frame .o.fxNPx

    radiobutton .o.oP -variable o_NP -value " "\
	    -text "PAL"
    radiobutton .o.oN -variable o_NP -value "-N"\
	    -text "NTSC"
    pack .o.oP .o.oN -side left -in .o.fxNPx -fill both -expand yes
    pack .o.lxNP .o.fxNPx -in .o.fxNP -anchor w -fill both -expand yes

    frame .o.fxo -bd 1 -relief raised
    label .o.lxo -text "MOD octaves"
    frame .o.fxox

    radiobutton .o.ooO -variable o_oO -value " "\
	    -text "Auto"
    radiobutton .o.oo -variable o_oO -value "-o"\
	    -text "1-3"
    radiobutton .o.oO -variable o_oO -value "-O"\
	    -text "All"
    pack .o.ooO .o.oo .o.oO -side left -in .o.fxox -fill x -expand yes
    pack .o.lxo .o.fxox -in .o.fxo -fill x -expand yes

    frame .o.fx56 -bd 1 -relief raised
    label .o.lx56 -text "Tempo"
    frame .o.fx56x

    radiobutton .o.o56 -variable o_56 -value " "\
	    -text "BPM" -command {
	.o.fl.el.ot configure -state normal
    }
    radiobutton .o.o5 -variable o_56 -value "-5"\
	    -text "PAL" -command {
	.o.fl.el.ot delete 0 end
	.o.fl.el.ot configure -state disabled
    }
    radiobutton .o.o6 -variable o_56 -value "-6"\
	    -text "NTSC" -command {
	.o.fl.el.ot delete 0 end
	.o.fl.el.ot configure -state disabled
    }
    pack .o.o56 .o.o5 .o.o6 -side left -in .o.fx56x -fill both -expand yes
    pack .o.lx56 .o.fx56x -in .o.fx56 -fill both -expand yes

    checkbutton .o.oT -variable o_oT -onvalue "-T" -offvalue " "\
	    -text "Tolerant loading"
    checkbutton .o.o0 -variable o_o0 -onvalue "-0" -offvalue " "\
	    -text "Ignore speed 0 stops"

    frame .o.fl -bd 1 -relief raised
    frame .o.fl.ll
    frame .o.fl.el

    mk_askoption .o.fl os "Speed"
    mk_askoption .o.fl ot "Tempo"
    mk_askoption .o.fl op "Start position"
    mk_askoption .o.fl oS "Stop position (p,l)"
    mk_askoption .o.fl om "Max playtime (sec)"
    mk_askoption .o.fl oc "Click removal"
    mk_askoption .o.fl oF "Mixing frequency"
    .o.fl.el.oF insert 0 "N/A"
    .o.fl.el.oF configure -state disabled
    pack .o.fl.ll .o.fl.el -side left -expand yes -fill y
}

proc mk_askoption {frame name label} {
    label $frame.ll.$name -text $label
    entry $frame.el.$name -relief sunken -width 8
    bind $frame.el.$name <Return> {focus .o}
    pack $frame.ll.$name -anchor w -expand yes
    pack $frame.el.$name -padx 5 -expand yes
}
