/*
 *  tk_screen.c - Handling of the screen using the Tcl/Tk/ET packages.
 *
 *  (C) 1994, 1995 Mikael Nordqvist (d91mn@efd.lth.se, mech@df.lth.se)
 */

#ifdef COMPILE_GUI_TK

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <tk.h>

#include "mod.h"

#include "../bmp/play.bmp"
#include "../bmp/stop.bmp"
#include "../bmp/next.bmp"
#include "../bmp/prev.bmp"
#include "../bmp/resize.bmp"
#include "../bmp/load.bmp"
#include "../bmp/mod_icon.bmp"

int tk_ok(void)
{
    return (getenv("DISPLAY") && *getenv("DISPLAY"));
}


void tk_init_screen(void)
{
    Tk_DefineBitmap(Et_Interp, Tk_GetUid("play"),
		    play_bits, play_width, play_height);
    Tk_DefineBitmap(Et_Interp, Tk_GetUid("stop"),
		    stop_bits, stop_width, stop_height);
    Tk_DefineBitmap(Et_Interp, Tk_GetUid("prev"),
		    prev_bits, prev_width, prev_height);
    Tk_DefineBitmap(Et_Interp, Tk_GetUid("next"),
		    next_bits, next_width, next_height);
    Tk_DefineBitmap(Et_Interp, Tk_GetUid("resize"),
		    resize_bits, resize_width, resize_height);
    Tk_DefineBitmap(Et_Interp, Tk_GetUid("load"),
		    load_bits, resize_width, load_height);
    Tk_DefineBitmap(Et_Interp, Tk_GetUid("mod_icon"),
		    mod_icon_bits, mod_icon_width, mod_icon_height);

    ET_INCLUDE(tk_setup.tcl);
    ET_INCLUDE(tk_screen.tcl);
    ET_INCLUDE(tk_loadscreen.tcl);
    ET_INCLUDE(tk_optionsscreen.tcl);
    ET_INSTALL_COMMANDS(tk_controller.c);

    if(GUI_size == 1)
	ET(global window_size; set window_size "chubby");
    else if(GUI_size == 2)
	ET(global window_size; set window_size "slim");
    else if(GUI_size == 3)
	ET(global window_size; set window_size "normal");
    else
	ET(global window_size; set window_size "verbose");

    if(!mixer_available()) {
	ET($w_volscale configure -state disabled
	   $w_mute configure -state disabled);
    }
    
    /* Pack the windows into '.' */
    ET(wm protocol . WM_DELETE_WINDOW {c_quit}
       wm iconbitmap . mod_icon
       wm iconname . Mod
       .m configure -cursor top_left_arrow
       set mod_version "%s(MOD_VERSION)"
       bind . <Enter> +{c_sync_volume}
       c_sync_volume
       update_window_size
       pack .m -expand yes -fill y
       update
       update idletasks);
}


void tk_cleanup_screen(void)
{
    ET(destroy .);
}


void tk_clear_screen(void)
{
}


void tk_print_songname(char *n)
{
    char noname[]="<no name>";

    if(n && !strlen(n))
	n=noname;
    ET(set m_song "%s(escape_name(n))");
}


void tk_print_filename(char *s)
{
    ET(set m_file "%s(escape_name(s))");
}


void tk_print_status(char *s)
{
    ET(set m_status "%s(escape_name(s))"
       update idletasks);
}


void tk_print_samples(int n)
{
    int i;
    struct sample_info *s;
    char buf[120];

    ET($w_samplelistbox delete 0 end);
    if(n == -1) {
	ET(set nr_samples 0);
	return;
    }

    for(i=1; i <= M.nr_samples; ++i) {
	s=&M.sample[i];
	sprintf(buf, "%2d %s %6d %3d   %s %+2d   %2s  %-6s",
		i, fill_string((M.format == MODFORMAT_ULT ? s->dosname :
				s->name), 27),
		s->length, s->volume,
		(s->looped ?
		 (s->looped == LOOP_BIDIR ? "BIDI" : "Yes ") : "    "),
		s->finetune, (s->bits_16 ? "16" : " 8"),
		s->length > 4 ? "mono" : "unused");
	ET($w_samplelistbox insert end "%s(escape_name(buf))");
    }
    ET(set nr_samples "%d(M.nr_samples)");
}


void tk_pre_load_update_screen(void)
{
    tk_clear_all_info();
    ET($w_viewcomment configure -state disabled
       $w_viewinfofile configure -state disabled);
    ET(.m configure -cursor watch);
    if(ET_INT(expr {$load_active}))
	ET(.l configure -cursor watch);
}


void tk_post_load_update_screen(void)
{
    ET(set songlength "%d(M.songlength)");	
    if(M.songtext)
	ET($w_viewcomment configure -state normal);
    if(M.infofile)
	ET($w_viewinfofile configure -state normal);
    /* The cursor is restored in tk_dim_buttons() */
}


static void print_fixed(char *s, int a, int b)
{
#if 0
    char buf[10];
    int tmp;
    
    strcpy(s, "   ");
    sprintf(buf, "%d", a);
    strcpy(&s[3-strlen(buf)], buf);
    strcat(s, " / ");
    sprintf(buf, "%d", b);
    tmp=strlen(buf);
    strcat(s, buf);
    strcpy(buf, "   ");
    buf[3-tmp]=0;
    strcat(s, buf);
#else
    sprintf(s, "%3d / %d", a, b);
#endif
}


void tk_print_pos(int pos, int line)
{
    char buf1[16], buf2[16], buf3[16];
    static int last_pos=-42;

    if(pos != -1) {
	print_fixed(buf1, pos, M.songlength-1);
	print_fixed(buf2, M.patterntable[pos], M.nr_patterns-1);
	print_fixed(buf3, line, 63);
	ET(set m_songpos "%s(buf1)"
	   set m_pattern "%s(buf2)"
	   set m_line "%s(buf3)");
    }
    else {
	ET(set m_songpos ""
	   set m_pattern ""
	   set m_line ""
	   change_songpos 0);
    }
    if(last_pos != pos)
	ET(set_songpos "%d(pos)");
    
    last_pos=pos;
}

void tk_print_speed(int spd, int tempo)
{
    if(spd != -1) {
	ET(set m_speed "%d(spd)"
	   set m_tempo "%d(tempo)");
    }
    else {
	ET(set m_speed ""
	   set m_tempo "");
    }
}


void tk_print_songinfo(int v, int samp, char *type, int sz, int filesz, int songlen, int pat)
{
    if(v != -1) {
	ET(set m_type "%s(type)"
	   set m_voices "%d(v)"
	   set m_samples "%d(samp)"
	   set m_size "%d(sz/1024)k"
	   set m_filesize "%d(filesz/1024)k");
    }
    else {
	ET(set m_type ""
	   set m_voices ""
	   set m_samples ""
	   set m_size ""
	   set m_filesize "");
    }
}    


void tk_print_time(int t)
{
    char buf[20];
    static int last_time;

    if(last_time == t)
	return;

    last_time=t;

    if(t != -1) {
	sprintf(buf, "%d:%02d", t/60, t%60);
	ET(set m_time "%s(buf)");
    }
    else
	ET(set m_time "");
}


void tk_clear_all_info(void)
{
    tk_print_samples(-1);
    tk_print_status("");
    tk_print_songname(" ");
    tk_print_filename("");

    tk_print_pos(-1,0);
    tk_print_speed(-1,0);
    tk_print_songinfo(-1, 0, 0, 0, 0, 0, 0);
    tk_print_time(-1);
    ET(update idletasks);
}


void tk_print_line(int songpos, int line)
{
#if 0
    static int foo=0;
    ET(change_samplepos "%d(foo%24)");
    ++foo;
    char buf[256];
    int pat, pos, v, step;

    if(!opt.dump)
        return;

    pat=M.patterntable[songpos];
    
    if(!line) {
        printf("\n\nSongpos [%3d/%3d]  Pattern [%3d/%3d]\n",
               songpos, M.songlength-1, pat, M.nr_patterns-1);
    }
    
    /* Remove hardwiring....... */
    sprintf(buf, "\n%02d  ", line);
    pos=5;
    for(v=0; v < 4 && v < M.nr_voices; ++v) {
        step=print_event(&buf[pos], GET_EVENT_PTR(v, pat, line), 4);
        pos+=step;
    }
    sprintf(&buf[pos], "|  %02d", line);
#endif
}

#endif /* COMPILE_GUI_TK */
