
######### Setup #########

# Cancel any -geometry requirements from the user

wm geometry . ""
wm minsize . 0 3
wm maxsize . 0 31

# Function to allow options to override defaults.

proc option_override {opt v} {
    upvar $v var
    if {[expr [string compare [set tmp [option get . $opt foo]] ""] != 0]} {
	set var $tmp
    }
}

# Set up default options.

set normalfont -*-times-bold-r-normal--*-140-*-*-p-*-*
set largefont -*-times-medium-r-normal--*-180-*-*-p-*-*
set fixfont -misc-fixed-bold-r-*-*-*-120-*-*-*-*-*-*

set fgcol Black
set bgcol SlateGray2
set activebgcol SlateGray3
set dimfgcol SlateGray
set lightfgcol Red
set selbgcol yellow2
set selfgcol Black

# Check resources for user specified options.

option_override color_foreground fgcol
option_override color_background bgcol
option_override color_activebackground activebgcol
option_override color_disabledforeground dimfgcol
option_override color_highlightforeground lightfgcol
option_override color_selectforeground selfgcol
option_override color_selectbackground selbgcol
option_override font_normal normalfont
option_override font_large largefont
option_override font_fixed fixfont

# Set up global variables

set mess_active 0
set load_active 0
set options_active 0
set mute_state 0
set browse_state 0
set random_state 0
set eat_double 0
set inserting 0

set songlength 30
set nr_files 0
set nr_dirs 0
set nr_listfiles 0

set first_filenr 0
set first_listnr 0

set prev_window_size foo42
set nr_visible_samples 8

# List of 
set ww_posscroll ""
set ww_next ""
set ww_prev ""
set ww_playstop ""

# Set up default widget settings

. configure -background $bgcol
option add *Background $bgcol
option add *Foreground $fgcol
option add *activeBackground $bgcol
option add *activeForeground $fgcol
option add *Font $normalfont
option add *disabledForeground $dimfgcol
option add *selectBackground $selbgcol
option add *selectForeground $selfgcol
#option add *Scrollbar.Foreground $bgcol
#option add *Scrollbar.Background $bgcol

option add *Checkbutton.Relief flat
option add *Radiobutton.Relief flat
option add *Checkbutton.selectColor $selfgcol
option add *Radiobutton.selectColor $selfgcol

option add *Menubutton.activeBackground $activebgcol
option add *Menubutton.borderWidth 0
option add *Menu.activeBackground $activebgcol
option add *Menu.selectColor $selfgcol

option add *Scrollbar.Foreground $bgcol
option add *Scrollbar.Background $bgcol

option add *Entry.Font $fixfont

option add *highlightThickness 0
option add *Menu.tearOff 0
option add *Button.padX 1
option add *Button.padY 1
option add *Menubutton.padY 1

option add *Listbox.selectMode extended

# Unpost menus when mousebutton is released

bind Menubutton <ButtonRelease-1> {tkMenuUnpost ""}

### Some support functions

# The following procedure shows a message in a toplevel window where the
# user only can press "Done".

proc show_message {wintitle message width} {
    global largefont

    toplevel .mess -relief raised -bd 2
    grab .mess
    focus .mess
    button .mess.b -font $largefont -text "Done" -pady 5\
	    -command {destroy .mess}
    message .mess.m -width $width -justify center -pady 10 -font $largefont\
	    -text $message
    
    wm title .mess $wintitle
    wm protocol .mess WM_DELETE_WINDOW {.mess.b invoke}
    wm resizable .mess no no
    pack .mess.m -fill x
    pack .mess.b -fill x -padx 8 -pady 8
    bind .mess <Return> {.mess.b invoke}
    bind .mess <Escape> {.mess.b invoke}
    tkwait window .mess
}

proc ask_left_right {wintitle message left right width} {
    global largefont

    toplevel .mess -relief raised -bd 2
    frame .mess.f
    button .mess.f.left -font $largefont -text $left -command {
	global ask_left_right_result
	set ask_left_right_result 0
	destroy .mess
    }
    button .mess.f.right -font $largefont -text $right -command {
	global ask_left_right_result
	set ask_left_right_result 1
	destroy .mess
    }
    message .mess.m -width $width -justify center -pady 10 -font $largefont\
	    -text $message
    
    wm title .mess $wintitle
    wm protocol .mess WM_DELETE_WINDOW {.mess.f.right invoke}
    wm resizable .mess no no
    pack .mess.f.left .mess.f.right -side left \
	    -expand yes -fill x -pady 5 -padx 5
    pack .mess.m -padx 10 -pady 5
    pack .mess.f -fill x

    focus .mess
    update idletasks
    grab .mess
    bind .mess <Return> {.mess.f.left invoke}
    bind .mess <Escape> {.mess.f.right invoke}
    tkwait window .mess
}

# Asks the user for a string.

proc ask_string {title mess defaultstring cursor_offset} {
    global fixfont largefont tcl2tcl

    toplevel .fe -relief raised -bd 2
    wm title .fe $title
    wm protocol .fe WM_DELETE_WINDOW {.fe.cancel invoke}
    wm resizable .fe no no
    grab .fe
    
    message .fe.text -justify center -aspect 350 -font $largefont -text $mess
    entry .fe.entry -relief sunken -width 40
    frame .fe.f
    button .fe.done -text "Done" -font $largefont -command {
	global tcl2tcl
	set a [.fe.entry get]
	destroy .fe; update; update idletasks; update
	set tcl2tcl $a
    }
    button .fe.cancel -text "Cancel" -font $largefont -command {
	global tcl2tcl
	destroy .fe; update; update idletasks; update
	set tcl2tcl ""
    }
    bind .fe.entry <Return> {.fe.done invoke}
    bind .fe.entry <Escape> {.fe.cancel invoke}
    bind .fe <Enter> {focus .fe.entry}
    
    .fe.entry insert 0 $defaultstring
    
    pack .fe.text -padx 10 -pady 5
    pack .fe.entry -padx 15 -pady 5
    pack .fe.done .fe.cancel -side left -in .fe.f\
	    -expand yes -fill x -pady 5 -padx 5
    pack .fe.f -fill x
    focus .fe.entry
    tkwait variable tcl2tcl
    return $tcl2tcl
}
